/// Josh's Logger
/// A Redacted Software Product
/// Developed & Maintained by Josh O'Leary
/// This work is dedicated to the public domain.


#pragma once

#include <format>
#include <string>
#include <iostream>
#include <chrono>
#include <Event.h>
#include <fstream>

namespace jlog
{
    enum class severity
    {
        none,
        verbose,
        debug,
        warning,
        error,
        fatal
    };

    struct LogEntry
    {
        severity level;
        std::string content;
        std::string timestamp;
    };



    static Event<LogEntry> on_log = Event<LogEntry>();

    std::vector<LogEntry> log_history;

    struct token
    {
        std::string colorCode = "";
        std::string content = "";
        std::string delimiter = "[]";
    };

    void log(std::vector<token> tokens);

    void info   (const std::string& message);
    void verbose(const std::string& message);

    void debug  (const std::string& message);

    void warning(const std::string& message);

    void error  (const std::string& message);

    void fatal  (const std::string& message);


    void info_spec(const std::string& message, const std::string& func, const std::string& file, int line);
    void verbose_spec(const std::string& message, const std::string& func, const std::string& file, int line);
    void debug_spec(const std::string& message, const std::string& func, const std::string& file, int line);
    void warning_spec(const std::string& message, const std::string& func, const std::string& file, int line);
    void error_spec(const std::string& message, const std::string& func, const std::string& file, int line);
    void fatal_spec(const std::string& message, const std::string& func, const std::string& file, int line);



}

#define INFO(i)    jlog::info_spec(i, __PRETTY_FUNCTION__, __FILE__, __LINE__);
#define VERBOSE(i) jlog::verbose_spec(i, __PRETTY_FUNCTION__, __FILE__, __LINE__);
#define DEBUG(i)   jlog::debug_spec(i, __PRETTY_FUNCTION__, __FILE__, __LINE__);
#define WARNING(i) jlog::warning_spec(i, __PRETTY_FUNCTION__, __FILE__, __LINE__);
#define ERROR(i)   jlog::error_spec(i, __PRETTY_FUNCTION__, __FILE__, __LINE__);
#define FATAL(i)   jlog::fatal_spec(i, __PRETTY_FUNCTION__, __FILE__, __LINE__);
